@@ -2,17 +2,29 @@ from __future__ import annotations
 import copy
 import re
 from schemy import Schema
+from datetime import date
+from datetime import datetime
 
 class Dataset(object):
 
-    def __init__(self, schema:Schema=None):
+    def __init__(self
+        , schema:Schema=None
+        , date_out_format='yyyy-MM-dd hh:mm:ss'):
         """
         """
         self.__set_schema(schema)
         self._data = []
         self._idx = 0
+        self._date_out_format = date_out_format
 
-    def __set_schema(self, schema:Schema=None):
+    def __iter__(self):
+        return self.rewind()
+
+    def __next__(self):
+        return self.next()
+
+    def __set_schema(self
+        , schema:Schema=None):
         """
         Private set method to schema.
         """
@@ -36,9 +48,10 @@ class Dataset(object):
         sizes = dict(zip(field_list.keys(), [0]*len(field_list)))
         #print(sizes)
         for row in self._data:
+            _row = self._parse_row(row)
             for f, p in field_list.items():
                 p = p - 1 # a posicao dos campos comecao com 0, nao 1
-                s = len(str(row[p]))
+                s = len(str(_row[p]))
                 if sizes[f]<s:
                     sizes[f] = s
         if rate>0:
@@ -49,15 +62,31 @@ class Dataset(object):
     def columns(self):
         return self._schema.get_names()
 
-    def to_str(self, length:int=None)->str:
+    def to_str(self, length:int=None, limit:int=None)->str:
+        """
+        length: length of each column
+        """
         fields = self._schema.get_names()
         if not fields:
             return '| * No fields * |'
-        if not length:
-            length = int(80/len(fields))
-        r = '| ' + ' | '.join([name.ljust(length, ' ')[:length] for name in fields]) + ' |\n'
-        for _, row in enumerate(self._data):
-            r += '| ' + ' | '.join([str(i).ljust(length, ' ')[:length] for i in row]) + ' |\n'
+        #
+        field_size = self.get_fields_size()
+        print(field_size)
+        #
+        width = []
+        for fname in fields:
+            name_width = len(fname)
+            val_width = field_size[fname]
+            if length and val_width>name_width:
+                val_width = 4 if length<4 else length
+            width.append(val_width if val_width>name_width else name_width)
+        #
+        r = '| ' + ' | '.join([name.ljust(width[j], ' ')[:width[j]] for j, name in enumerate(fields)]) + ' |\n'
+        for i, row in enumerate(self._data):
+            _row = self._parse_row(row)
+            if limit and i>=limit:
+                break
+            r += '| ' + ' | '.join([str(val).ljust(width[j], ' ')[:width[j]] for j, val in enumerate(_row)]) + ' |\n'
         return r
 
     def get_schema(self):
@@ -66,6 +95,17 @@ class Dataset(object):
         """
         return self._schema
 
+    def _insert_list(self, row:list, idx:int=None):
+        """
+        """
+        if len(row)!=len(self._schema.get_names()):
+            # print((len(row), len(self._schema.get_names()), row))
+            raise Exception("Número de colunas a inserir difere do número de colunas do schema definido.")
+        if not idx:
+            self._data.append(copy.copy(row))
+        else:
+            self._data[idx] = copy.copy(row)
+
     def _insert(self, row, idx:int=None)->Dataset:
         """
         Insere dados no dataset.
@@ -81,6 +121,8 @@ class Dataset(object):
             row = nrow
             self._insert(row, idx)
         elif type(row) is list:
+            self._insert_list(row)
+            """
             if len(row)!=len(self._schema.get_names()):
                 print((len(row), len(self._schema.get_names()), row))
                 raise Exception("Número de colunas a inserir difere do número de colunas do schema definido.")
@@ -88,10 +130,11 @@ class Dataset(object):
                 self._data.append(copy.copy(row))
             else:
                 self._data[idx] = copy.copy(row)
+            """
         elif type(row) is tuple:
             self._insert(list(row), idx)
         else:
-            print(type(row))
+            # print(type(row))
             raise Exception("Tipo não reconhecido.")
         return self
 
@@ -148,13 +191,14 @@ class Dataset(object):
 
     def add_field(self
         , name:str
-        , ftype:str=str
+        , ftype=str
         , size:int=50
         , default=None
         , primary_key:bool=False
         , auto_increment:int=0
         , col_ref:int=None
         , pos='a'
+        , out_format:str=None
         , optional:bool=True):
         self._schema.add_field(name=name
             , ftype=ftype
@@ -164,13 +208,14 @@ class Dataset(object):
             , auto_increment=auto_increment
             , col_ref=col_ref
             , pos=pos
-            , optional=optional)
+            , optional=optional
+            , out_format=out_format)
         pos = self._schema.get_field_pos(name)
         if not pos:
             raise Exception("Error creating new field '{}'.".format(name))
         pos = pos - 1
         for _, row in enumerate(self._data):
-            row[pos:pos] = [default]
+            row[pos:pos] = [None] # [default]
         return self
 
     def remove_col(self, col):
@@ -189,6 +234,7 @@ class Dataset(object):
         destruir o original.
         """
         cp = Dataset(copy.deepcopy(self._schema))
+        cp._date_out_format = self._date_out_format
         if copy_data:
             cp._data = copy.deepcopy(self._data)
         return cp
@@ -236,15 +282,16 @@ class Dataset(object):
         if self._idx<0:
             return None
         try:
-            return self._transform_row(self._data[self._idx])
+            return self._parse_row(self._data[self._idx])
         except:
-            return None
+            raise StopIteration
 
     def next(self):
         """
         """
+        v = self.current()
         self._idx += 1
-        return self.current()
+        return v
 
     def prev(self):
         """
@@ -266,10 +313,52 @@ class Dataset(object):
         self._data = []
         return self
 
-    def _transform_row(self, row):
+    def readrow(self, idx:int):
         """
         """
-        return row
+        row = self._data[idx]
+        return self._parse_row(row)
+
+    def __date_out_format_convert(self, date_out_format:str):
+        """
+        The regular dataformat (SQL data format) is
+        not the same of datetime object in Python.
+        This method is to convert the SQL format to
+        Python format, so it can be used to convert
+        datatime objects value to string.
+        @see _parse_row()
+        """
+        date_out_format = date_out_format.replace('yyyy', '%Y').\
+            replace('MM', '%m').\
+            replace('dd', '%d').\
+            replace('hh', '%H').\
+            replace('mm', '%M').\
+            replace('ss', '%S')
+        return date_out_format
+
+    def _parse_row(self, row:list):
+        """
+        Method to out lines parsing None values
+        and casting other types to string.
+        """
+        parsed_row = []
+        for i, col in enumerate(row):
+
+            ftype = self._schema._schema[i]._f['ftype']
+            default = self._schema._schema[i]._f['default']
+            out_format = self._schema._schema[i]._f['out_format']
+
+            # if value is null use default value
+            col = default if col is None else col
+
+            # se tiver algum valor
+            if col is not None:
+                # se for do tipo datetime
+                if ftype in [datetime, date]:
+                    out_format = self._date_out_format if out_format is None else out_format
+                    col = col.strftime(self.__date_out_format_convert(out_format))
+            parsed_row.append(col)
+        return parsed_row
 
     def where(self, filter):
         filter_result = self.filter(filter)
@@ -280,6 +369,15 @@ class Dataset(object):
         # return new_dataset
         return self
 
+    def all(self):
+        """
+        Get all data. Parsed. Without any filter or order.
+        """
+        all_data = []
+        for row in self._data:
+            all_data.append(self._parse_row(row))
+        return all_data
+
     def filter(self, filter)->list:
         """
         equal usa-se um dicionário: {'[campo]':2}
@@ -296,13 +394,14 @@ class Dataset(object):
         filter_result = [True]*len(self._data)
         fpos = self._schema.get_all_field_pos()
         for i, row in enumerate(self._data):
+            row = self._parse_row(row)
             if type(filter) is dict:
                 for k, v in filter.items():
                     mod = ''
                     # se a chave for string
                     if type(k) is str:
                         parsed = parse_name(k)
-                        print(parsed)
+                        # print(parsed)
                         if parsed:
                             k = parsed[0]
                             mod = parsed[1].lower()
